

##################################
priorAlpha12 = function(alpha1, model.select = TRUE){
#return the prior distribution of alpha_1, this function can be modified to different priors. 
	result = log(1 / alpha1)
    return (result)
}

priorAlpha22 = function(alpha2, model.select = TRUE){
#return the prior distribution of alpha_2, this function can be modified to different priors. 
	result = log(1 / alpha2)    
	return (result)
}

priorT0 = function(t, model.select = TRUE){
#return the prior distribution of t_0, this function can be modified to different priors. 
	result = dlnorm(t, meanlog = 4.25, sdlog = .5612, log = T)   
    return (result)
}


priorB = function(b, model.select = TRUE){
#return the prior distribution of b, this function can be modified to different priors. 
    result = dnorm(b, b.mean, b.sigma, log = T)
    return (result)
}

priorSigma2 = function(s){
#return the prior distribution of s2, this function can be modified to different priors. 
    return (1 / s)
}

###############################################
#########prior distribution for model 1########
priorAlpha11 = function(alpha1, model.select = TRUE){
	result = log(1 / alpha1)
    return (result)
}

priorAlpha21 = function(alpha2, model.select = TRUE){
	result = log(1 / alpha2)    
	return (result)
}

priorSigma1 = function(s){
    return (1 / s)
}

###############################################
######density function for model 4#############
densityFunc.m4 = function(y, t, alpha1, alpha2, t0, b, sigmaSquare){
    t1 = t[t<=t0]
    y1 = y[t<=t0]
    t2 = t[t>t0]
    y2 = y[t>t0]
    m1 = alpha1*exp(-alpha2/t1)
    m2 = alpha1*exp(-alpha2/t0) - b * t0 + b * t2
    L1 = - sum((y1 - m1)^2)
    L2 = - sum((y2 - m2)^2)
    L = (L1+L2) / 2 / sigmaSquare - length(y) / 2 * log(sigmaSquare)
    return(L)
}

###selection of model#############

posterior.draw.2 = function(y, t, theta.2, sigmaSquare.2){
	alpha1.current = theta.2[1]
	alpha2.current = theta.2[2]
	t0.current = theta.2[3]
	b.current = theta.2[4]
	sigmaSq.current = sigmaSquare.2
	t0.current = drawT0.2(y, t, alpha1.current, alpha2.current, t0.current, b.current, sigmaSq.current, delta.t0)
	alpha2.current = drawAlpha2.2(y, t, alpha1.current, alpha2.current, t0.current, b.current, sigmaSq.current, delta.alpha2)
	b.current = drawB.2(y, t, alpha1.current, alpha2.current, t0.current, b.current, sigmaSq.current, delta.b)
	alpha1.current = drawAlpha1.2(y, t, alpha1.current, alpha2.current, t0.current, b.current, sigmaSq.current, delta.alpha1)
	sigmaSq.current = drawSigmaSq.2(y, t, alpha1.current, alpha2.current, t0.current, b.current, sigmaSq.current)
	theta.2.new = c(alpha1.current, alpha2.current, t0.current, b.current)
	return (list(theta.2.new, sigmaSq.current))
}

##################################
drawT0.2 = function(y, t, alpha1, alpha2, t0, b, sigmaSquare, delta.t0){
    t0New = rnorm(1, t0, delta.t0)
    density.1 = densityFunc.m4(y, t, alpha1, alpha2, t0, b, sigmaSquare) + priorT0(t0)
    density.2 = densityFunc.m4(y, t, alpha1, alpha2, t0New, b, sigmaSquare) + priorT0(t0New)
    tNew = ifelse(runif(1) < (exp(density.2 - density.1)), t0New, t0)
    tNew = ifelse(tNew < tmax, tNew, t0)
    return (tNew)
}


drawAlpha2.2 = function(y, t, alpha1, alpha2, t0, b, sigmaSquare, delta.alpha2){
    alpha2New = rnorm(1, alpha2, delta.alpha2)
    density.1 = densityFunc.m4(y, t, alpha1, alpha2, t0, b, sigmaSquare) + (priorAlpha22(alpha2))
    density.2 = densityFunc.m4(y, t, alpha1, alpha2New, t0, b, sigmaSquare) + (priorAlpha22(alpha2New))
    alphaNew = ifelse(runif(1) < exp(density.2 - density.1), alpha2New, alpha2)
    return (alphaNew)
}


drawB.2 = function(y, t, alpha1, alpha2, t0, b, sigmaSquare, delta.b){
    bNew = rnorm(1, b, delta.b)
    density.1 = densityFunc.m4(y, t, alpha1, alpha2, t0, b, sigmaSquare) + priorB(b)
    density.2 = densityFunc.m4(y, t, alpha1, alpha2, t0, bNew, sigmaSquare) + priorB(bNew)
    bNew = ifelse(runif(1) < (exp(density.2 - density.1)), bNew, b)
    return (bNew)
}

drawAlpha1.2 = function(y, t, alpha1, alpha2, t0, b, sigmaSquare, delta.alpha1){
    alpha1New = rnorm(1, alpha1, delta.alpha1)
    density.1 = densityFunc.m4(y, t, alpha1, alpha2, t0, b, sigmaSquare) + priorAlpha12(alpha1)
    density.2 = densityFunc.m4(y, t, alpha1New, alpha2, t0, b, sigmaSquare) + priorAlpha12(alpha1New)
    alphaNew = ifelse(runif(1) < (exp(density.2 - density.1)), alpha1New, alpha1)
    return (alphaNew)
}


drawSigmaSq.2 = function(y, t, alpha1, alpha2, t0, b, sigmaSquare){
    t1 = t[t<=t0]
    y1 = y[t<=t0]
    t2 = t[t>t0]
    y2 = y[t>t0]
    m1 = alpha1*exp(-alpha2/t1)
    m2 = alpha1*exp(-alpha2/t0) - b * t0 + b * t2
    L1 = sum((y1 - m1)^2)
    L2 = sum((y2 - m2)^2)
    betaConst = (L1+L2) / 2
    alphaConst = (length(y) + 2)/2
    gammaVariable = rgamma(1, shape = alphaConst)
    sigmaSquareNew = betaConst/gammaVariable
    return (sigmaSquareNew)
}


###################################################################
informationMatrix.m4<-function(theta, sigmaSq, t){
	alpha = theta[1]
	beta = theta[2]
	t0 = theta[3]
	b = theta[4]
    aa<-array(NA,c(length(t), 4))
    aa[, 1]<-ifelse(t<t0, exp(-beta/t), exp(-beta/t0))
    aa[, 2]<-ifelse(t<t0, exp(-beta/t)*(-alpha/t), -alpha/t0 * exp(-beta/t0))
    aa[, 3]<-ifelse(t<t0,0,exp(-beta/t0)*alpha*beta/t0^2 - b)
    aa[, 4]<-ifelse(t<t0, 0, t - t0)
    matrix = t(aa) %*% (aa) / sigmaSq
    return (matrix)
}



##################################

posterior.draw = function(a1.m4, a2.m4, t0.m4, b.m4, N.sample = 10000){
	theta.1.sample = array(NA, c(N.sample, 2))
	theta.2.sample = array(NA, c(N.sample, 4))
	sigmaSquare.1.sample = sigmaSquare.2.sample = array(NA, N.sample)
	theta.2.current = c(a1.m4, a2.m4, t0.m4, b.m4)
	sigma.2.sample = array(NA, N.sample)
	sigma.2.current = sigma.m4
	for(iter in 1:N.sample){
		print (iter)
		po.2 = posterior.draw.2(y, t, theta.2.current, sigma.2.current)
		theta.2.current = po.2[[1]]
		sigma.2.current = po.2[[2]]
		theta.2.current[3] = ifelse(theta.2.current[3] > 205, 205, theta.2.current[3])
		theta.2.sample[iter, ] = theta.2.current
		sigma.2.sample[iter] = sigma.2.current
	}
	return (list(theta.2.sample, sigma.2.sample))
}


